from string import Template

from IPython.display import display, HTML
try:
    from google.colab.output import eval_js
except:
    def eval_js(js):
        body = HTML(f"""<script>
        {js}
        </script>""")
        display(body)

from .rythmbox_html import (JAVASCRIPT, HEAD,
            HTML_FRAME, COL_TEMPLATE, MCOL_TEMPLATE, ROW_TEMPLATE)

colors = ["red", "yellow", "green", "blue",
          "lime", "purple", "orange"]

display(HTML(HEAD))

def make_cols(an_id, klass, cols, data, tmpl):
    col_html = ""
    for i in range(cols):
        t = Template(tmpl)
        k_addition = ""
        if i in data:
            k_addition = " on"
        part = t.substitute(**{"an_id":an_id, "klass":klass+k_addition, "pos":i})
        col_html += part
    return col_html


def show_frame(data, cols, rows=1):
    display(HTML(HEAD))

    rt = Template(ROW_TEMPLATE)

    # インジケーターを作る
    col_html = make_cols("i", "indicator", cols, [], MCOL_TEMPLATE)
    mk_html = rt.substitute(**{"cols":col_html})

    # ブロックを作る
    bl_html = ""
    for r in range(rows):
        col_html = make_cols("g", "button "+colors[r], cols, data[r], COL_TEMPLATE)
        bl_html += rt.substitute(**{"cols":col_html})

    jst = Template(JAVASCRIPT)
    js = jst.substitute(**{"speed":250, "cols": cols})

    frame = Template(HTML_FRAME)
    args = {"script":js, "markers":mk_html,
            "blocks":bl_html, "cols": cols}
    display(HTML(frame.substitute(**args)))


def rb(timings):
    if isinstance(timings, list):
        if isinstance(timings[0], int):
            timing_list = [x for x in timings if x < 8]
            show_frame([timing_list], 8)
        elif isinstance(timings[0], str):
            timing_list = [[], [], [], []]
            for ti, ts in enumerate(timings):
                for i, s in enumerate(ts):
                    if s == "1":
                        timing_list[ti].append(i)
            show_frame(timing_list, 8, 4)

    elif isinstance(timings, str):
        timing_list = [n for n, x in enumerate(timings[:8]) if x == "1"]
        show_frame([timing_list], 8)


def rb2(timing_str_list):
    timing_str = timing_str_list[:8]
    timings = [[], [], [], []]
    for i in range(0, len(timing_str)):
        for n, s in enumerate(words):
            if s in timing_str[i].lower():
                timings[n].append(i)
    show_frame(timings, 8, 4)


    



from IPython.display import display, HTML

from .tictactoe_html import CSS, FRAME

cell_tmpl = """
<span class="order">{0}</span>
<span class="{1}">{2}</span>
"""


def show_board(gl):
    cell = [""]*9

    for n in range(1, 10):
        if f"c{n}" in gl:
            c = cell_tmpl.format(n, "pc", "X")
            cell[gl[f"c{n}"]-1] = c
        if f"p{n}" in gl:
            c = cell_tmpl.format(n, "human", "O")
            cell[gl[f"p{n}"]-1] = c

    if "c" in gl:
        for n, pl in enumerate(gl["c"]):
            c = cell_tmpl.format(n, "pc", "X")
            cell[pl-1] = c

    if "p" in gl:
        for n, pl in enumerate(gl["p"]):
            c = cell_tmpl.format(n, "human", "O")
            cell[pl-1] = c

    html_body = CSS
    html_body += FRAME.format(cell)
    display(HTML(html_body))





# rpg_html.py
#
# rpgの実行時に表示する，HTMLやCSS，JavaSCriptなどを集めたモジュール
# 
# Copyright (c) 2021 Atsush Shibata(shivata@m-info.co.jp)
#
# Released under the MIT license.

HTML_FRAME = """
 <link href="https://fonts.googleapis.com/css?family=M+PLUS+Rounded+1c&amp;subset=japanese" rel="stylesheet">
 <link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons">
 <style>

.box {
    font-family: 'M PLUS Rounded 1c', sans-serif;
    font-size: 14pt;
    color : #ffffff;
    text-shadow : 2px 2px 2px rgba(51, 51, 51, 0.8);
    background-color : #0033cc;
    background : linear-gradient(#3366ff, #0033cc, #000066);
    padding : 12px 16px;
    box-shadow :
        3px 3px 5px rgba(51, 51, 51, 0.7) inset,
        0 0 2px 3px #666666,
        0 0 0   6px #ffffff,
        0 0 2px 8px #666666;
    width: 90%;
    margin: auto;
    margin-top: 8px;
    margin-bottom: 8px;
}

#box dt {
    font-weight: bold;
    color: #e84;
    margin: 0.4em 0 0 0;
    font-size: 16pt;
}

#box dd {
    font-weight: bold;
    padding: 0.1em 0;
}

.button {
    font-size: 12pt;
    color: black;
    background-color: white;
}


</style>
<div>
  <div id='box_$body_id' class='box'>
  </div>
</div>
<script>
function add_message(m, e) {
    var elem = document.getElementById('box_$body_id');
    var child = document.createElement(e);
    child.innerHTML     = m;
    elem.appendChild(child)
}

function add_rawhtml(h) {
    var elem = document.getElementById('box_$body_id');
    elem.innerHTML += h;
}

</script>
"""

INITIAL_OUTPUT = """
<script src="https://cdnjs.cloudflare.com/ajax/libs/howler/2.2.3/howler.min.js"></script>
<script>
var death = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/deth.mp3?raw=true'],
    preload: true,
    html5: true
});

var hero_emerge = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/hero_emerge.mp3?raw=true'],
    preload: true,
    html5: true
});

var hit1 = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/hit1.mp3?raw=true'],
    preload: true,
    html5: true
});

var hit2 = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/hit2.mp3?raw=true'],
    preload: true,
    html5: true
});

var monster_emerge = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/monster_emerge.mp3?raw=true'],
    preload: true,
    html5: true
});

var success = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/success.mp3?raw=true'],
    preload: true,
    html5: true
});

var status_sound = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/status.mp3?raw=true'],
    preload: true,
    html5: true
});

var fight_music = new Howl({
    src: ['https://github.com/shibats/mpb_samples/blob/main/assets/fight_music.mp3?raw=true'],
    preload: true,
    html5: true
});
/*Howler.autoUnlock = true;*/

</script>

"""



SCRIPT = """
// constants
var wo_width = 800,
    wo_height = 300,
    ball_x = 66,
    ball_y = -10,
    ball_r = 10,
    ball_height = 5,
    wall_x = 720,
    wall_y = wo_height,
    wall_w = 60,
    wall_h = 30,
    wall_color1 = 'brown',
    wall_color2 = 'gray',
    fl2_width = wall_w+20,
    fl2_height = -ball_y,
    cannon_x = 10,
    cannon_y = 460,
    cannon_w = 40,
    cannon_h = 20,
    with_sound = false,
    breakable = true;


make_world = function(add_wall, delta) {
	setup();
    load_sound();
    
	//Add a floor
	var floor = Matter.Bodies.rectangle(wo_width/2, wo_height, wo_width, 10, {
        label: 'floor',
		density:5000.0,
		friction: 80,
        restitution: 900,
		isStatic: true, //An immovable object
		render: {
			visible: true
		}
	});
	Matter.World.add(world, floor);

	var floor2 = Matter.Bodies.rectangle(wall_x+delta-10, wo_height, fl2_width, -fl2_height, {
        label: 'floor',
		density:5000.0,
		friction: 80,
        restitution: 900,
		isStatic: true, //An immovable object
		render: {
			visible: true
		}
	});
	//Matter.World.add(world, floor2);



    walls = add_wall(delta);
	
	Matter.Events.on(engine, 'collisionStart', function(event) {
	var pairs = event.pairs;

    if( pairs[0].bodyB.label == 'ball') {
         var bx = pairs[0].bodyB.velocity.x, by = pairs[0].bodyB.velocity.y,
             bv = bx**2+by**2;
         if( bv > 60 && breakable ){
         
            if( pairs[0].bodyA.label.startsWith('wall')) {
                var tb = pairs[0].bodyA, ox = tb.bounds.min.x, oy = tb.bounds.min.y,
                    vx = tb.velocity.x*20, vy = tb.velocity.y*20;
                var i = 0;
                for( var x = 0; x < 3; x++ ) {
                    for( var y = 0; y < 3; y++ ) {

                        var wall = make_wall(ox+x*(wall_h/3), oy+y*(wall_h/3),
                                            wall_w/3.5, wall_h/3.5, 300, 's_wall'+i, wall_color1);
                        Matter.World.add(world, wall);
                        walls.push(wall);             
                        i ++;   
                    }
                }
                Matter.World.remove(world, pairs[0].bodyA);
                pairs[0].bodyB.label = 'ball_col'
                if( with_sound ) {
                    heavy_hit.play();
                }
            } else {
                if( with_sound ) {
                    light_hit.play();
                    breakable = false;
                }
            }
         } else {
             if( with_sound ) {
                 light_hit.play();
                breakable = false;
             }
             pairs[0].bodyA.label = 'ball_col'
         }
	 }
	});

	//Start the engine
	Matter.Engine.run(engine);
	Matter.Render.run(render);

};

var load_sound = function() {
    blast = new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/blast.mp3?raw=true'],
        preload: true,
        html5: true
    });

    light_hit = new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/light_hit.mp3?raw=true'],
        preload: true,
        html5: true,
        volume: 0.3
    });
    
    heavy_hit = new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/heavy_hit.mp3?raw=true'],
        preload: true,
        html5: true,
        volume: 0.25
    });

};

var setup = function() {
    //Fetch our canvas
	var canvas = document.getElementById('world');

	//Setup Matter JS
	engine = Matter.Engine.create();
	world = engine.world;
	body = Matter.Body;
	render = Matter.Render.create({
		canvas: canvas,
		engine: engine,
		options: { 
			width: wo_width,
			height: wo_height,
			background: 'transparent',
			wireframes: false,
			showAngleIndicator: false
		}
	});

};


var make_wall = function(x, y, w, h, density, label, color ) {
    var wall = Matter.Bodies.rectangle(x, y, w, h, { 
        label: label,
        density: density,
        friction: 1,
        frictionAir: 0.01,
        restitution: 0.001,
        render: {
            fillStyle: color,
            strokeStyle: 'black',
            lineWidth: 1
        }
    });
    return wall
}

add_wall1 = function(delta) {
    //Add a wall
	var walls = [];
	for(var i=0; i <= 5; i++ ) {
        var wall = make_wall(wall_x, wall_y-i*wall_h, wall_w, wall_h, 800, 'wall'+i, wall_color1);
		Matter.World.add(world, wall);
		walls.push(wall);
	}
    return walls;
};

add_wall2 = function(delta) {
    //Add a wall
	var walls = [];
	for(var i=0; i <= 5; i++ ) {
        var col = wall_color2, label = 'hwall'+i;
        if( i == 1 ) {
            col = wall_color1
            label = 'wall'+i;
        }
        var wall = make_wall(wall_x+delta, wall_y-i*wall_h, wall_w, wall_h, 800, label, col);
		Matter.World.add(world, wall);
		walls.push(wall);
	}
    return walls;
};


var bang = function(powder) {
    var but = document.getElementById('button');
    but.disabled = true;
    var but2 = document.getElementById('button2');
    but2.disabled = true;


    //Add a ball
    var ball = Matter.Bodies.circle(ball_x, ball_y+wo_height-ball_r-ball_height, ball_r, {
        label: 'ball',
        density:2000.0,
        friction: 0.99,
        frictionAir: 0.0002,
        restitution: 0.0001,
        render: {
            fillStyle: '#222',
            strokeStyle: 'black',
            lineWidth: 1
        }
    });
    var v = powder/10;
    body.setVelocity(ball, {x:v, y:-v});
    if( with_sound ) { 
        blast.play();
    }

    Matter.World.add(world, ball);

};

var sound_on = function() {
  with_sound = true;
};

"""

LOAD_SCRIPT = """
	<script>

var script = document.createElement( 'script' );

script.type = 'text/javascript';
script.src = 'https://cdnjs.cloudflare.com/ajax/libs/matter-js/0.17.1/matter.min.js';

var firstScript = document.getElementsByTagName( 'script' )[ 0 ];
firstScript.parentNode.insertBefore( script, firstScript );
	</script>

    <script src="https://cdnjs.cloudflare.com/ajax/libs/howler/2.2.3/howler.min.js" type="text/javascript"></script>
"""


HTML_TEMPLATE = """
        <script src="https://cdnjs.cloudflare.com/ajax/libs/matter-js/0.17.1/matter.min.js" type="text/javascript"></script>
		<script src="https://cdnjs.cloudflare.com/ajax/libs/howler/2.2.3/howler.min.js" type="text/javascript"></script>
		<link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons">
		<style>
#cannon {
	background-image: url('https://github.com/shibats/mpb_samples/blob/main/assets/cannon2.png?raw=true');
	background-size: cover;
	width: 80px;
	height: 80px;
	margin-top: -76px;
	margin-left: 8px;
	z-index: -10;
}

#countdown {
    margin-top: -100px;
    margin-left: 2em;
}

.container {
}


.world_outer {
  margin-left: 0;
  width: 800px;
}

.button_wrapper {
	text-align: center;
}

button {
  font-size: 14pt;
  margin: 0.5em 0;
  text-align: center:
}

@media screen and (max-width:900px) {
  .container {
      -webkit-transform: scale(0.75);
      transform: scale(0.75);
      margin-left: -6em;
      margin-top: -1em;
  }
}

@media screen and (max-width: 500px) {
  .container {
      -webkit-transform: scale(0.4);
      transform: scale(0.4);
      margin-left: -6em;
      margin-top: -1em;
  }
}

@media screen and (max-width: 300px) {
  .container {
      -webkit-transform: scale(0.2);
      transform: scale(0.2);
      margin-left: -6em;
      margin-top: -1em;
  }
}


@media screen and (max-width: 200px) {
  .container {
      -webkit-transform: scale(0.25);
      transform: scale(0.25);
      margin-left: -6em;
      margin-top: -3em;
  }
}

		</style>
    <div class="container">
        <div class="button_wrapper">
            <button id="button" onclick="countdown(); sound_on();"><span class="material-icons">volume_up</span>音ありでうつ</button>
            <button id="button2" onclick="countdown();"><span class="material-icons">volume_off</span>音なしでうつ</button>
            <h2>火薬量: ${powder} ${dist_display}</h2>
       </div>
		<div class="world_outer">
            <canvas id="world"></canvas>
            <div id="cannon"></div>
            <h2 id="countdown">--</h2>
		</div>
    </div>
	<script>
        ${SCRIPT}

        make_world(add_wall${stage}, ${delta});
        var countdown = function() {
            setTimeout(function() {bang(${powder}*3, true);}, 2000);
            var countdown = 3;
            var cd = function() {
                countdown--;
                var elem = document.getElementById('countdown');
                elem.innerHTML = countdown;
            };
            cd();
            setTimeout(cd, 1000);
            setTimeout(cd, 2000);
        };

	</script>
"""


JAVASCRIPT = """
var position = 0, speed = 200, blocks = ${cols},
    sound = {'red':'crave', 'yellow':'kick', 'green':'snare', 'blue':'hat', 'lime':'hat', 'purple':'tom', 'orange':'crap' },
    make_sound = false;
if( !timer_id ) {
    var timer_id = null;
}

var load_sound = function() {
    sounds = {
    'clap' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/clap.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'kick' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/kick.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'snare' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/snare.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'cymbal' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/cymbal.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'hat' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/hat.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'tom' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/tom.wav?raw=true'],
        preload: true,
        html5: true,
    }),

    'crave' : new Howl({
        src: ['https://github.com/shibats/mpb_samples/blob/main/assets/rithmmachine/clave.wav?raw=true'],
        preload: true,
        html5: true
    }),

    }

    
};


var getelements = function(classname) {
    var elem = document.getElementsByClassName(classname);
    var array = Array.prototype.slice.call(elem);
    return array;
}

var proceed = function() {
    oldelem = getelements('pos'+position);
    for( var item in oldelem ){
        oldelem[item].classList.remove('tick');
    };

    position = (position+1)%blocks;
    elem = getelements('pos'+position);
    for( var item in elem ){
        elem[item].classList.add('tick');
        console.log(make_sound)
        if( make_sound && elem[item].classList.value.includes(' on') ) {
            var s = sound[elem[item].classList[1]];
            sounds[s].play();
        }
    };

};

var periodic = function() {
    proceed();
    timer_id = setTimeout(periodic, speed);
};

var sound_on = function() {
    make_sound = true;
}

var sound_off = function() {
    make_sound = false;
}

load_sound();

if( timer_id ) {
    clearTimeout(timer_id);
    timer_id = null;
}

periodic();
"""

HEAD = """
  <script src="https://cdnjs.cloudflare.com/ajax/libs/howler/2.2.3/howler.min.js"></script>
  <link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons">
"""

HTML_FRAME = """
  <style>
.container {
  display: block;
  margin: auto;

}

.indicator {
  margin: 0;
  display: inline-block;
  height:2em;
  width: 30px;
  color: white;
  text-align: center;
  background-color: #888;
  border: 2px solid #fff;
  border-radius: 4px;
  
}

.button {
  margin: 0;
  display: inline-block;
  height: 30px;
  width: 30px;
  border: 2px solid #fff;
  border-radius: 4px;
}

.red {
  background-color: #5c0011;
}

.yellow {
  background-color: #614700;
}

.green {
  background-color:#092b00;
}

.blue {
  background-color: #002766;
}

.lime {
  background-color: #254000;
}

.purple {
  background-color: #120338;
}

.orange {
  background-color: #612500;
}



.tick {
  border: 2px solid #fe8;
}

.indicator.tick {
  background-color: #eee;
}

.red.tick  {
  background-color: #a8071a;
}

.yellow.tick {
  background-color: #ad8b00;
}

.green.tick {
  background-color: #237804;
}

.blue.tick {
  background-color: #0050b3;
}

.lime.tick {
  background-color: #5b8c00;
}

.purple.tick {
  background-color: #391085;
}

.orange.tick {
  background-color: #ad4e00;
}



.red.on  {
  background-color: #cf1322;
}

.yellow.on {
  background-color: #d4b106;
}

.green.on {
  background-color: #389e0d;
}

.blue.on {
  background-color: #096dd9;
}

.red.on.tick  {
  background-color: #ff7875;
}

.yellow.on.tick {
  background-color: #fff566;
}

.green.on.tick {
  background-color: #95de64;
}

.blue.on.tick {
  background-color: #69c0ff;
}

.lime.on.tick {
  background-color: #d3f261;
}

.purple.on.tick {
  background-color: #b37feb;
}

.orange.on.tick {
  background-color: #ffc069;
}


  </style>

  <div class="container">
    <button id="sound_on" onclick="sound_on();"><span class="material-icons">volume_up</span>音を鳴らす</button>
    <button id="sound_off" onclick="sound_off();"><span class="material-icons">volume_off</span>音を止める</button>

    ${markers}

    ${blocks}

  </div>

  <script>
  ${script}
  </script>

"""

MCOL_TEMPLATE = """
      <div id="${an_id}_${pos}" class="${klass} pos${pos}">
        ${pos}
      </div>
"""

COL_TEMPLATE = """
      <div id="${an_id}_${pos}" class="${klass} pos${pos}">
      </div>
"""


ROW_TEMPLATE = """
    <div class="row">
      ${cols}
    </div>
"""



# sugoiパッケージ用の初期化スクリプト
# 
# Copyright (c) 2021 Atsush Shibata(shivata@m-info.co.jp)
#
# Released under the MIT license.

from datetime import datetime
from json import loads
import requests
from bs4 import BeautifulSoup
import numpy as np


def doomsday_clock(year):
    dc_url = "https://ja.wikipedia.org/wiki/%E4%B8%96%E7%95%8C%E7%B5%82%E6%9C%AB%E6%99%82%E8%A8%88"
    resp = requests.get(dc_url)
    soup = BeautifulSoup(resp.text, 'html.parser')

    secs = {}

    table = [x for x in soup.find_all("table", class_="wikitable")]
    for n, tr in enumerate(table[0].find_all("tr")):
        if n > 1:
            tds = [x for x in tr.find_all("td")]
            mstr = tds[1].text.replace("前", "")
            smstr = mstr.split("分")
            sec = 0
            if smstr[1]:
                sec = int(smstr[0])*60+int(smstr[1].replace("秒", ""))
            else:
                sec = int(smstr[0])*60
            secs[int(tds[0].text.replace("年", ""))] = sec

    if year in secs:
        return secs[year]
    else:
        return "不明"


def get_number_of_space_satellite():
    resp = requests.get("https://www.ucsusa.org/media/11490")
    lines = len(resp.text.split("\n"))-1
    return(lines)


def stock_price_prediction():
    """
    ランダムな[-1, 1]のうちランダムに数値を返す
    """
    return choice([-1, 1])


def get_wf_json(code=13):
    url = f"https://www.jma.go.jp/bosai/forecast/data/forecast/{code:02}0000.json"
    resp = requests.get(url)
    d = loads(resp.text)
    return d



def pp(code=13):
    """
    codeにある地域(デフォルトは東京)の
    直近の降水確率を返す
    """
    d = get_wf_json(code)
    return int(d[0]["timeSeries"][1]["areas"][0]["pops"][0])


def temp(code=13):
    """
    codeにある地域(デフォルトは東京)の
    直近の気温予測を返す
    """
    d = get_wf_json(code)
    c = int(d[0]["timeSeries"][2]["areas"][0]["temps"][0])
    return int(1.8*c+32)


def f2c(f):
    """
    華氏(Fahrenheit)から摂氏(Celsius)に変換
    """
    return int((f-32)*(5/9))


def ppl(code=13):
    """
    codeにある地域(デフォルトは東京)の直近の降水確率を返す
    6時間ごと，4つ
    """
    d = get_wf_json(code)
    return [int(x) for x in d[0]["timeSeries"][1]["areas"][0]["pops"]][:4]


def get_eq_info():
    """
    api.p2pquake.netから，日本で発生した直近の地震の情報を取り出す
    戻り値は辞書
    """
    # 地震の情報を読み込む
    url = "https://api.p2pquake.net/v2/jma/quake?limit=1"
    r = requests.get(url)
    ql = loads(r.text)
    dt = ql[0]["created_at"]
    dto = datetime.strptime(ql[0]["created_at"].split(".")[0], "%Y/%m/%d %H:%M:%S")
    retd = ql[0]["earthquake"]["hypocenter"]
    retd["datetime"] = dto
    return retd


coef = np.array([0.02975907, -1.47542641, 19.58163225, 21.7626419])


def test_score(h):
    """
    勉強時間に対する，テストの点数を返す
    100点満点，10時間で限界効用逓減によって下がって行く。
    """
    if h > 20:
        h = 20
    score = int(np.poly1d(coef)(h))+1
    if score > 100:
        score = 100
    return score


def test_score2(h, i):
    if h > 20:
        h = 20
    score = np.poly1d(coef)(h)+1-20
    score = int(score*(i/100)+20)
    if score > 100:
        score = 100
    return score


from string import Template
from random import randint
from math import floor

from IPython.display import display, HTML
try:
    from google.colab.output import eval_js
    HTML_fook = HTML
    display_hook = display


except:
    from IPython.display import IFrame
    from IPython.display import FileLink
    def eval_js(js):
        body = HTML(f"""<script>
        {js}
        </script>""")
        display(body)


    def HTML_fook(body):
        f = open('./tmp.html', 'w')
        f.write(body)
        f.close()


    def display_hook(html):
        display(IFrame('./tmp.html', width="100%", height="400px"))

DELTA = 0

def set_delta():
    global DELTA
    while not DELTA:
        DELTA = randint(-5, 5)*5


def load_script():
    html_body = HTML_fook(LOAD_SCRIPT)
    display_hook(html_body)


from .cannon_html import HTML_TEMPLATE, SCRIPT, LOAD_SCRIPT

def cannon_game(powder):
    html_source = Template(HTML_TEMPLATE)
    args = {"SCRIPT":SCRIPT, "stage":1, "powder":powder, "delta": 0,
            "dist_display": ""}
    html_body = HTML_fook(html_source.substitute(**args))
    display_hook(html_body)


def cannon_game2(powder):
    html_source = Template(HTML_TEMPLATE)
    args = {"SCRIPT":SCRIPT, "stage":2, "powder":powder, "delta": 0,
            "dist_display": ""}
    html_body = HTML_fook(html_source.substitute(**args))
    display_hook(html_body)


def cannon_game3(powder, delta=-1):
    global DELTA
    if delta != -1:
        DELTA = delta
    else:
        set_delta()
    html_source = Template(HTML_TEMPLATE)
    args = {"SCRIPT":SCRIPT, "stage":2, "powder":powder, "delta": DELTA,
            "dist_display": "距離 : "+str(450+DELTA)}
    html_body = HTML_fook(html_source.substitute(**args))
    display_hook(html_body)

    DELTA = 0


def get_distance():
    set_delta()
    return DELTA+450


def calc_powder(distance):
    p = ((50*(distance+10))/(9*2.643))**0.5
    return floor(p*1000)/1000



# rpg.py
#
# RPG風のユニット管理，表示をするシンプルなクラス
# Colaboratory，またはJupyter上での利用を想定
# 
# Copyright (c) 2021 Atsush Shibata(shivata@m-info.co.jp)
#
# Released under the MIT license.


from string import Template
from random import random
from time import sleep

from IPython.display import display, HTML
try:
    from google.colab.output import eval_js
except:
    def eval_js(js):
        body = HTML(f"""<script>
        {js}
        </script>""")
        display(body)

from .rpg_html import HTML_FRAME, INITIAL_OUTPUT
FRAME_TMPL = Template(HTML_FRAME)

# ユーティリティ関数

def load_audio():
    """
    音声ファイルを事前ロードするための関数
    """
    body = HTML(INITIAL_OUTPUT)
    display(body)


def play_audio(the_id):
    """
    IDを指定して音声を再生するJavaScriptを出力する関数
    """
    display(HTML(f"""<script>
    {the_id}.play();
    </script>
    """))


def pause_audio(the_id):
    """
    音声を止めるJavaScriptを出力する関数
    """
    display(HTML(f"""<script>
        {the_id}.stop();
        </script>
    """))


load_audio()  # モジュールインポート時に音声を事前ロードする


def play_with_button(sound_id, playable,
                     play_title="音を出す",
                     stop_title="音を消す"):
    """
    「音を鳴らす」ボタンを表示する
    """
    if playable:
        play_audio(sound_id)
    else:
        rawhtml = (f"""add_rawhtml('<button """
            f"""class="button" onclick="javascript:{sound_id}.play();">"""
            f"""<span class="material-icons">volume_up</span>"""
            f"""{play_title}</button>　');""")
        rawhtml += (f"""add_rawhtml('<button """
            f"""class="button" onclick="javascript:{sound_id}.stop();">"""
            f"""<span class="material-icons">volume_off</span>"""
            f"""{stop_title}</button>'); console.log('foo&');""")
        eval_js(rawhtml)



def add_message(message, elem="p"):
    """
    JavaScriptをinvokeして出力にメッセージを追加するユーティリティ関数
    """
    eval_js(f"add_message('{message}', '{elem}')")



class ユニット:
    """
    RPG風のシンプルなユニットクラス
    """

    playable = False
    body_id = 0
    play_title = "音を出す"
    stop_title = "音を消す"

    # メッセージ類
    msg_emerge = Template("$kindがうまれた。")

    msg_status = "ステータス"
    msg_occupation = "職業　"
    msg_hp = "体力　"
    msg_power = "パワー"

    msg_dead = Template("$kindは死んでいるので戦えない。")
    msg_t_dead = Template("$kindは死んでいる。これ以上はかわいそうだ。")
    mgs_fight = Template("$kindと$t_kindが戦った。")
    msg_hit = Template("$kindが$t_kindに$damage_targetのダメージを与えた。")
    msg_beat = Template("$kindは$t_kindを倒した！")
    msg_beaten = Template("$kindは$t_kindに倒されて死んでしまった。")


    def __init__(self, kind):
        """
        初期化メソッド
        
        初期化時に，音を鳴らすHTMLを出力して，表示中のブラウザが
        音を鳴らせるかどうか判別して，フラグを管理する
        """
        try:
            # ブラウザが音を慣らせるかどうかを判別
            display(HTML(f"""
        <audio id="nosound" preload>
        <source src="https://github.com/shibats/mpb_samples/blob/main/assets/nosound.mp3?raw=true" type="audio/mp3">
        </audio>
            """))
            eval_js(f"""
                    var bgm1 = document.querySelector("#nosound");
                    bgm1.play();
                    """)
        except:
            self.__class__.playable = False

        # ステータス表示のエレメントIDを初期化

        # ステータスを初期化
        self.kind = kind
        self.体力 = 0
        self.パワー = 0
        # 音声を先読みするため，HTMLを出力
        load_audio()

        # kindが「戦士」かどうかで，出現音声を分ける
        sound_id = "monster_emerge"
        if kind == "戦士":
            sound_id = "hero_emerge"

        # メッセージを表示
        display(HTML(FRAME_TMPL.substitute(body_id=self.body_id)))
        play_with_button(sound_id, self.playable,
                         self.play_title, self.stop_title)
        add_message(self.msg_emerge.substitute(kind=self.kind))
        self.__class__.body_id += 1


    def ステータス(self):
        """
        ステータスを表示する
        """

        load_audio()

        display(HTML(FRAME_TMPL.substitute(body_id=self.body_id)))
        self.__class__.body_id += 1

        play_with_button("status_sound", self.playable,
                         self.play_title, self.stop_title)

        add_message(self.msg_status, "dt")
        add_message(self.msg_occupation+f" : {self.kind: >5}", "dd")
        add_message(self.msg_hp+f" : {self.体力: >5}", "dd")
        add_message(self.msg_power+f" : {self.パワー: >5}", "dd")


    def 戦う(self, target):
        """
        ユニットのインスタンスtargetと戦う
        """
        load_audio()
        display(HTML(FRAME_TMPL.substitute(body_id=self.body_id)))
        self.__class__.body_id += 1

        if self.体力 <= 0:
            # 体力がない場合
            add_message(self.msg_dead.substitute(kind=self.kind))
            play_with_button("death", self.playable,
                             self.play_title, self.stop_title)
            return

        if target.体力 <= 0:
            # 戦う相手の体力がない場合
            add_message(self.msg_t_dead.substitute(kind=target.kind))
            play_with_button("death", self.playable,
                             self.play_title, self.stop_title)
            return

        # 双方のダメージを乱数で決める
        damage_mine = int(target.パワー*(random()/2+0.5))
        damage_target = int(self.パワー*(random()/2+0.5))
        self.体力 -= damage_mine
        target.体力 -= damage_target

        play_with_button("fight_music", self.playable,
                         self.play_title, self.stop_title)
        msg = self.mgs_fight.substitute(kind=self.kind, t_kind=target.kind)
        add_message(msg)
        sleep(2)

        # こちらの攻撃
        play_audio("hit1")
        msg = self.msg_hit.substitute(kind=self.kind,
                                      t_kind=target.kind,
                                      damage_target=damage_target)
        add_message(msg)
        sleep(1)

        if target.体力 <= 0:
            # 勝った場合
            pause_audio("fight_music")
            play_audio("success")
            msg = self.msg_beat.substitute(kind=self.kind,
                                           t_kind=target.kind)
            add_message(msg)
            return

        # 相手の攻撃
        msg = self.msg_hit.substitute(kind=target.kind,
                                      t_kind=self.kind,
                                      damage_target=damage_mine)
        add_message(msg)
        play_audio("hit1")
        sleep(1)

        if self.体力 <= 0:
            # 負けた場合
            pause_audio("fight_music")
            play_audio("death")
            msg = self.msg_beaten.substitute(kind=self.kind,
                                             t_kind=target.kind)
            add_message(msg)


class Unit(ユニット):
    """
    クラス ユニット の英語版
    """
    playable = False
    body_id = 100

    play_title = "Play"
    stop_title = "Stop"

    # メッセージ類
    msg_emerge = Template("$kind has born.")

    msg_status = "Status"
    msg_occupation = "Occupation　"
    msg_hp =         "Hitpoint  　"
    msg_power =      "Strength    "

    msg_dead = Template("$kind has been already dead.")
    msg_t_dead = Template("$kind had died.")
    mgs_fight = Template("$kind fought with $t_kind.")
    msg_hit = Template("$kind gave $damage_target damage to $t_kind.")
    msg_beat = Template("$kind beat $t_kind !")
    msg_beaten = Template("$kind  has been defeated by $t_kind.")


    def __init__(self, kind):
        self.kind = kind
        self.hitpoint = 0
        self.strength = 0
        # kindが「Hero」かどうかで，出現音声を分ける
        sound_id = "monster_emerge"
        if kind.lower() == "hero":
            sound_id = "hero_emerge"

        # メッセージを表示
        display(HTML(FRAME_TMPL.substitute(body_id=self.body_id)))
        play_with_button(sound_id, self.playable,
                         self.play_title, self.stop_title)
        add_message(self.msg_emerge.substitute(kind=self.kind))
        self.__class__.body_id += 1


    def copy_status_to(self):
        """
        hitpointを体力に，Strengthをパワーにコピーする
        """
        self.体力 = self.hitpoint
        self.パワー = self.strength


    def copy_status_from(self):
        """
        体力をにhitpoint，パワーをStrengthにコピーする
        """
        self.hitpoint = self.体力
        self.strength = self.パワー


    def status(self):
        """
        ステータスを表示する
        """
        self.copy_status_to()
        self.ステータス()


    def fight(self, target):
        self.copy_status_to()
        target.copy_status_to()
        self.戦う(target)
        self.copy_status_from()
        target.copy_status_from()




CSS = """
<style>
.container_t {
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  background: #eee;
}

.play-area {
  display: grid;
  width: 300px;
  height: 300px;
  grid-template-columns: auto auto auto;
}

.block {
  display: flex;
  position: relative;
  width: 100px;
  height: 100px;
  align-items: center;
  justify-content: center;
  font-size: 48pt;
  font-weight: bold;
  border: 3px solid black;
  transition: background 0.2s ease-in-out;
}

.block .number {
  font-size: 32pt;
  color: #bbb;
  position: absolute;
  top: 1.2em;
  left: 1.4em;
  z-index: 0;
}

.human {
  color: #482;
  z-index: 2;
}

.pc {
  color: #c42;
  z-index: 2;
}

.order {
  color: #46d;
  font-size: 18pt;
  position: absolute;
  top: 0.4em;
  left: 0.7em;
  z-index: 3;
}



#block_0,
#block_1,
#block_2 {
  border-top: none;
}

#block_0,
#block_3,
#block_6 {
  border-left: none;
}

#block_6,
#block_7,
#block_8 {
  border-bottom: none;
}

#block_2,
#block_5,
#block_8 {
  border-right: none;
}

.draw {
  color: orangered;
}
</style>
"""

FRAME = """
    <div class="container_t">
        <div class="play-area">
          <div id="block_0" class="block">
            {0[0]}
            <span class="number">1</span>
          </div>
          <div id="block_1" class="block pc">
            {0[1]}
            <span class="number">2</span>
          </div>
          <div id="block_2" class="block">
            {0[2]}
            <span class="number">3</span>
          </div>
          <div id="block_3" class="block">
            {0[3]}
            <span class="number">4</span>
          </div>
          <div id="block_4" class="block">
            {0[4]}
            <span class="number">5</span>
          </div>
          <div id="block_5" class="block">
            {0[5]}
            <span class="number">6</span>
          </div>
          <div id="block_6" class="block">
            {0[6]}
            <span class="number">7</span>
          </div>
          <div id="block_7" class="block">
            {0[7]}
            <span class="number">8</span>
          </div>
          <div id="block_8" class="block">
            {0[8]}
            <span class="number">9</span>
          </div>
        </div>
    </div>
"""


# rpgパッケージ用の初期化スクリプト
# 
# Copyright (c) 2021 Atsush Shibata(shivata@m-info.co.jp)
#
# Released under the MIT license.

import os
from io import BytesIO
import io
import numpy as np
from imageio import imread
import scipy
from scipy.io import wavfile
import IPython


def load(tone):
    """
    wavファイルを読み込む関数。toneにはc,e,gを指定。
    """
    if tone not in ["c", "d", "e", "f", "g", "a", "b"]:
        print("引数は，c，d，eのどれかの文字列にしてください。")
        return
    base = os.path.dirname(__file__)
    wavpath = os.path.join(base, f"./wavs/{tone}.wav")
    with open(wavpath, 'rb') as wav:
        freq, snd_arr = wavfile.read(wav)
        return snd_arr.astype(dtype="int").tolist()
    return 


def play(the_list):
    return IPython.display.Audio(np.array(the_list), rate=22050)




from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='mpb_lib',
    packages=find_packages(),
    package_data={'': ['sound/wavs/c.wav',
                       'sound/wavs/d.wav',
                       'sound/wavs/e.wav',
                       'sound/wavs/f.wav',
                       'sound/wavs/g.wav',
                       'sound/wavs/a.wav',
                       'sound/wavs/b.wav' ]},

    version='0.1.54', # バージョン

    license='MIT', # ライセンス

    install_requires=[],

    author='Atsushi Shibata',
    author_email='shibata@m-info.co.jp',

    url='',

    description='A library for minpro.',
    long_description='',
    long_description_content_type='text/markdown',
    keywords='',

    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
)